import { Profile, Message } from '@line/bot-sdk'

declare global {
    type LiffInitSuccessCallback = (
        data: {
            language: string,
            context: {
                type: string,
                viewType: string,
                userId: string,
                utouId: string,
                roomId: string,
                groupId: string,
            }
        }
    ) => void
    
    type LiffInitErrorCallback = (error: Error) => void
    
    type LIFFPlugins = 'bluetooth'
    
    interface LINEBluetoothRequestDeviceFilter {
        deviceId: string
    }
    
    interface LIFFRequestDeviceOptions {
        filters: LINEBluetoothRequestDeviceFilter[]
    }
    
    interface BluetoothRemoteGATTCharacteristic {
        service?: BluetoothRemoteGATTService
        uuid: string
        value: DataView
        readValue(): Promise<DataView>
        writeValue(value: BufferSource): Promise<void>
        startNotifications(): Promise<BluetoothRemoteGATTCharacteristic>
        stopNotifications(): Promise<BluetoothRemoteGATTCharacteristic>
        addEventListener(eventName: string, event: EventListener): void
        removeEventListener(eventName: string): void
    }
    
    interface BluetoothRemoteGATTService {
        device: BluetoothDevice
        uuid: string
        getCharacteristic(characteristicUUID: string): Promise<BluetoothRemoteGATTCharacteristic>	
    }
    
    interface BluetoothRemoteGATTServer {
        device: BluetoothDevice
        connected: boolean
        connect(): Promise<BluetoothRemoteGATTService>
        disconnect(): void
        getPrimaryService(serviceUUID: string): Promise<BluetoothRemoteGATTService>
    }
    
    interface BluetoothDevice {
        id: string
        name?: string
        gatt?: any
        watchingAdvertisements: boolean
        watchAdvertisements(): Promise<void>
        unwatchAdvertisements(): void
        addEventListener(eventName: string, event: EventListener): void
        removeEventListener(event: EventListener)
    }
    
    interface LIFF {
        init(
            initSuccessCallback: LiffInitSuccessCallback,
            errorCallback: LiffInitErrorCallback,
        ): void
        openWindow: ({
            url: string,
            external?: boolean
        })
        getAccessToken(): string
        getProfile(): Promise<Profile>
        sendMessages(messages: Message[]): Promise<void>
        closeWindow(): void
        initPlugins(plugins: LIFFPlugins[]): Promise<void>
        bluetooth: {
            getAvailability(): Promise<boolean>
            requestDevice(options?: LIFFRequestDeviceOptions): Promise<BluetoothDevice>
        }
    }
    
    const liff: LIFF;
}

